/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * $Id: JOnASConfigurationGenerator.java 5290 2009-09-14 12:23:49Z alitokmen $
 */
package org.ow2.jasmine.monitoring.installer.jonas;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.ow2.jonas.tools.configurator.Jonas;
import org.ow2.jonas.tools.configurator.api.JonasConfigurator;

/**
 * @author Julien Legrand
 */
public class JOnASConfigurationGenerator {

    /**
     *
     */
    private final Jonas jonas;

    /**
     * The actual configurator for the JOnAS server.
     */
    private JonasConfigurator configurator = null;

    /**
    *
    */
    private VelocityEngine ve = null;

    /**
    *
    */
    private VelocityContext context = null;

    /**
    *
    */
    private Template template = null;

    /**
     *
     */
    private static final String STARTJM_BAT_NAME_TEMPLATE = "startjm.bat.vm";

    /**
    *
    */
    private static final String STARTJM_SH_NAME_TEMPLATE = "startjm.sh.vm";

    /**
   *
   */
    private static final String STARTJM_BAT_NAME = "startjm.bat";

    /**
  *
  */
    private static final String STARTJM_SH_NAME = "startjm.sh";

    /**
     * Constructor
     * 
     * @param jonasRoot
     * @param jonasBase
     */
    public JOnASConfigurationGenerator(final String jonasRoot, final String jonasBase) {
        super();

        jonas = new Jonas(jonasRoot);
        configurator = jonas.getJonasConfigurator();
        configurator.setJonasBase(jonasBase);

        // lines to find the template even within the jar file
        final Properties p = new Properties();
        p.setProperty("resource.loader", "class");
        p.setProperty("class.resource.loader.class",
                "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");

        ve = new VelocityEngine();
        context = new VelocityContext();

        try {
            ve.init(p);
            final StringWriter writer = new StringWriter();

            context.put("jonas_root", jonasRoot);
            context.put("jonas_base", jonasBase);

            template = ve.getTemplate(STARTJM_BAT_NAME_TEMPLATE);

            if (template != null) {
                template.merge(context, writer);
            }
            write(jonasBase + File.separator + STARTJM_BAT_NAME);

            template = ve.getTemplate(STARTJM_SH_NAME_TEMPLATE);

            if (template != null) {
                template.merge(context, writer);
            }
            write(jonasBase + File.separator + STARTJM_SH_NAME);

        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 
     */
    public final void configure() {

        configurator.setServices("jtm,db,dbm,security,resource,ejb3,jaxws,web,ear,depmonitor");

        // add a topic named jasmine for EoS
        // add a topic named JASMINeAlarmsTopic for the notifications export system
        configurator.setJmsTopics("jasmine,JASMINeAlarmsTopic");

        // add a JAAS entry for EoS authentication
        try {
//            configurator.setJaasEntry("eos");
        } catch (final Exception ignored) {
            System.out.println("The JOnAS version you choose doesn't support JAAS configuration.");
            System.out.println("You need to configure it manually.");
            System.out.println("Add this entry in the JONAS_BASE/conf/jaas.config file : ");

            final StringBuffer buffer = new StringBuffer();
            buffer.append("eos{");
            buffer.append("\n");
            buffer.append("\t");
            buffer.append("org.ow2.jonas.security.auth.spi.JResourceLoginModule required");
            buffer.append("\n");
            buffer.append("\t");
            buffer.append("resourceName=\"memrlm_1\"");
            buffer.append("\n");
            buffer.append("\t");
            buffer.append(";");
            buffer.append("};");

            System.out.println(buffer.toString());
        }

        // execute
        configurator.execute();

        System.out.println("A JOnAS base has been created and configured for JASMINe Monitoring.");

    }

    /**
     * Write generated configuration in the file "filename"
     * 
     * @param filename
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */

    private void write(final String filename) throws Exception {
        final File file = new File(filename);
        if (!file.exists()) {
            file.createNewFile();
        }

        final BufferedWriter out = new BufferedWriter(new FileWriter(filename));
        out.write(generate());
        out.close();
    }

    /**
     * @return
     * @throws Exception
     * @throws ParseErrorException
     * @throws ResourceNotFoundException
     */
    private String generate() throws Exception {

        final StringWriter writer = new StringWriter();

        if (template != null) {
            template.merge(context, writer);
        }
        return new String(writer.getBuffer());
    }
}
